"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNodes = exports.prepareNodes = void 0;
const tslib_1 = require("tslib");
const gatsby_node_helpers_1 = tslib_1.__importDefault(require("gatsby-node-helpers"));
const pluralize_1 = tslib_1.__importDefault(require("pluralize"));
const HELPERS = gatsby_node_helpers_1.default({
    typePrefix: `ServerlessAurora`,
});
function flattenQueryResultsToNodeList(sourceNodeArgs, defaultDatabaseName, queryResults) {
    const { createContentDigest } = sourceNodeArgs;
    const nodeList = [];
    for (const { query, rows } of queryResults) {
        const idFieldName = query.idFieldName || `id`;
        const nodeType = HELPERS.generateTypeName(query.nodeName);
        for (const row of rows) {
            const rawId = String(row[idFieldName]);
            nodeList.push({
                nodeData: {
                    id: rawId,
                    type: query.nodeName,
                    row,
                    links: { parents: {}, children: {} },
                    internal: {
                        type: nodeType,
                        content: JSON.stringify(row),
                        contentDigest: createContentDigest(row),
                    },
                },
                metadata: {
                    rawId,
                    nodeId: HELPERS.generateNodeId(query.nodeName, rawId),
                    query,
                    row,
                },
            });
        }
    }
    return nodeList;
}
function findParentNodes(parentNodeName, parentMatcher, childNode, nodeList) {
    if (!parentNodeName && !parentMatcher)
        return [];
    let parentNodes = parentNodeName
        ? nodeList.filter(node => node.metadata.query.nodeName === parentNodeName)
        : nodeList;
    if (parentMatcher) {
        parentNodes = parentNodes.filter(parentNode => parentMatcher(childNode.metadata.row, parentNode.metadata.row));
    }
    return parentNodes;
}
function addForignKeyRelationships(nodeList) {
    for (const childNode of nodeList) {
        const { nodeName: childNodeName, parentNodeName, parentMatcher } = childNode.metadata.query;
        const parentNodes = findParentNodes(parentNodeName, parentMatcher, childNode, nodeList);
        if (!parentNodes.length)
            continue;
        const childToParentLinks = [];
        const parentToChildKey = `${pluralize_1.default(childNodeName)}___NODE`;
        parentNodes.forEach(parentNode => {
            var _a;
            const childToParentKey = `${pluralize_1.default(parentNode.metadata.query.nodeName)}___NODE`;
            childToParentLinks.push([childToParentKey, parentNode.metadata.nodeId]);
            parentNode.nodeData.links.children[parentToChildKey] = (_a = parentNode.nodeData.links.children[parentToChildKey]) !== null && _a !== void 0 ? _a : [];
            parentNode.nodeData.links.children[parentToChildKey].push(`${childNode.metadata.nodeId}`);
        });
        childToParentLinks.forEach(childToParentLink => {
            var _a;
            const [childToParentKey, parentNodeId] = childToParentLink;
            childNode.nodeData.links.parents[childToParentKey] = (_a = childNode.nodeData.links.parents[childToParentKey]) !== null && _a !== void 0 ? _a : [];
            childNode.nodeData.links.parents[childToParentKey].push(`${parentNodeId}`);
        });
    }
}
function prepareNodes(sourceNodeArgs, defaultDatabaseName, queryResults) {
    const nodeList = flattenQueryResultsToNodeList(sourceNodeArgs, defaultDatabaseName, queryResults);
    addForignKeyRelationships(nodeList);
    return nodeList;
}
exports.prepareNodes = prepareNodes;
function createNodes(sourceNodeArgs, nodeList) {
    const { actions, reporter } = sourceNodeArgs;
    reporter.info(`Creating ${nodeList.length} node(s)`);
    nodeList.forEach(node => {
        const NodeFactory = HELPERS.createNodeFactory(node.metadata.query.nodeName);
        const gatsbyNode = NodeFactory(node.nodeData);
        actions.createNode(gatsbyNode);
    });
}
exports.createNodes = createNodes;
